import argparse

from .service import run
from .setup import setup

parser = argparse.ArgumentParser(prog="CleanEmonPopulator", description="The CLI for CleanEmon-Populator")
subparsers = parser.add_subparsers(help='commands')

# Service
populate_parser = subparsers.add_parser("service", help="Run a service")
populate_parser.add_argument("service_name", action="store", choices=["populate"])

# Script
script_parser = subparsers.add_parser("script", help="Run a script")
script_parser.add_argument("script_name", action="store", choices=["setup"])

args = parser.parse_args()
print(args)

if args.service_name:
    if args.service_name == "populate":
        run()

elif args.script_name:
    if args.script_name == "setup":
        setup()



import pytest

from CleanEmonPopulator.buffer import AutoBuffer
from CleanEmonCore.models import EnergyData


@pytest.mark.skip
def test_buffer():
    a = AutoBuffer(10)
    data = EnergyData("test", [{"delete_me": 34}])
    for i in range(10):
        print(i)
        a.append_data(data, "06580f5a9b5b888e3a0ad8fb8a035bda")
    print("Done")
    for i in range(10):
        print(i)
        a.append_data(data, "06580f5a9b5b888e3a0ad8fb8a035bda")
    print("done")



from . import CONFIG_FILE

from CleanEmonCore.CouchDBAdapter import CouchDBAdapter
from CleanEmonCore.models import EnergyData


class AutoBuffer:
    def __init__(self, capacity):
        self.db_adapter = CouchDBAdapter(CONFIG_FILE)
        self.data = {}
        self._capacity = capacity
        self._count = 0

    def _append_hook(self):
        cleared = False

        self._count += 1
        if self._count >= self._capacity:
            cleared = self._clear()

        return cleared

    def _clear(self):
        for document, data_list in self.data.copy().items():
            print(f"Sending {len(data_list)} records to {document}...")
            self.db_adapter.append_energy_data(*data_list, document=document)
            del self.data[document]
        self._count = 0

        return bool(self.data)  # Empty dictionary == False, otherwise True

    def append_data(self, energy_data: EnergyData, document: str):
        if document not in self.data:
            self.data[document] = []

        self.data[document].append(energy_data)

        cleared = self._append_hook()

        return cleared



"""A simple integration set of utilities connecting EmonPi's ecosystem with CouchDB"""

import os

from CleanEmonCore.dotfiles import get_dotfile

from .EmonPiAdapter import EmonPiAdapter

_CONFIG_FILENAME = "clean.cfg"
_SCHEMA_FILENAME = "schema.json"

# Check if config-file lies in directory of execution
# If not, consider working with the global one
if os.path.exists(_CONFIG_FILENAME):
    CONFIG_FILE = os.path.abspath(_CONFIG_FILENAME)
else:
    CONFIG_FILE = get_dotfile(_CONFIG_FILENAME)

# Check if schema-file lies in directory of execution
# If not, consider working with the global one
if os.path.exists(_SCHEMA_FILENAME):
    SCHEMA_FILE = os.path.abspath(_SCHEMA_FILENAME)
else:
    SCHEMA_FILE = get_dotfile(_SCHEMA_FILENAME)



import configparser


def generate_config(config_file):
    config = configparser.ConfigParser(interpolation=None)
    config["Emon"] = {}

    # EmonPi Section
    print("--- Configure EmonPi ---")

    # IP
    default = "127.0.0.1"
    temp = input(f"IP of EmonPi ({default}): ")
    emon_ip = temp if temp else default

    # Bearer Credentials
    bearer_credentials = input("EmonPi Bearer Credentials (only the hash): ")
    if not bearer_credentials:
        bearer_credentials = "REPLACE_ME"
        print("Bearer Credentials cannot be omitted. This will cause trouble!")
        print(f"Please refer to http://{emon_ip}/feed/api, look up for the API-Key")
        print("and replace it in the generated config file.")
        input("Press enter to continue...")

    config["Emon"]["endpoint"] = f"http://{emon_ip}"
    config["Emon"]["bearer_credentials"] = bearer_credentials

    with open(config_file, "w", encoding="utf8") as f_out:
        config.write(f_out)

    print(f"Config file was successfully generated at: {config_file}")



import os

from CleanEmonPopulator import CONFIG_FILE
from CleanEmonPopulator import SCHEMA_FILE


def test_files():
    assert os.path.exists(CONFIG_FILE)
    assert os.path.isfile(CONFIG_FILE)

    assert os.path.exists(SCHEMA_FILE)
    assert os.path.isfile(SCHEMA_FILE)



"""Module intended to be run as an integrated app entry"""

from datetime import datetime

from CleanEmonCore.CouchDBAdapter import CouchDBAdapter
from CleanEmonCore import CONFIG_FILE as DB_CONFIG_FILE
from CleanEmonCore.models import EnergyData

from . import CONFIG_FILE as EMON_CONFIG_FILE
from . import SCHEMA_FILE
from .buffer import AutoBuffer

from .EmonPiAdapter import EmonPiAdapter

from CleanEmonCore.Events import Timer
from CleanEmonCore.Events import Observer


class Reporter(Observer):
    """Class used to implement the fetch-update-send cycle, as long as other
    important integration utilities.
    """

    def __init__(self, timer, *, log_to_screen=False):
        super().__init__(timer)

        self.log_to_screen = log_to_screen
        self.emon_adapter = EmonPiAdapter(EMON_CONFIG_FILE, schema_file=SCHEMA_FILE)
        self.db_adapter = CouchDBAdapter(DB_CONFIG_FILE)
        self.current_document_id = None
        self.current_date = None
        self.buffer = AutoBuffer(60)

    def on_notify(self, *args, **kwargs):
        """This function implements the fundamental fetch-update-send loop.
        As a *side effect*, it checks whether the date has changed and updates
        the document_id as needed.
        """

        self._check_date_change()
        data = self.emon_adapter.fetch_data()
        energy_data = EnergyData(date=self.current_date, energy_data=[data])
        status = self.buffer.append_data(energy_data, document=self.current_document_id)

        if self.log_to_screen:
            print(self.current_document_id)
            print(self.current_date)
            print(data)
            print(f"Uploaded: {status}")

    def _check_date_change(self):
        """If the date has changed since the last check, this means that either a
        new document must be created (usual case) or another document must be
        appended and not recreated (rare case, occurs in development).

        This function keeps
            - self.current_document_id
            - self.current_date
        up to date according to the current date.
        """

        temp = datetime.now()
        date = temp.strftime("%Y-%m-%d")

        if date != self.current_date:
            if self.log_to_screen:
                print("Changed date")
            document_id = self.db_adapter.get_document_id_for_date(date)
            if not document_id:
                energy_data = EnergyData(date=date)
                document_id = self.db_adapter.create_document(initial_data=energy_data)

            self.current_document_id = document_id
            self.current_date = date


def run():
    cooldown_timer = Timer(5)
    Reporter(cooldown_timer, log_to_screen=True)
    cooldown_timer.run()



from .. import CONFIG_FILE
from .. import SCHEMA_FILE

from .generate_config import generate_config
from .generate_schema import generate_schema


def setup():
    generate_config(CONFIG_FILE)
    generate_schema(CONFIG_FILE, SCHEMA_FILE)
